part of disposable;

/// Provides a stand-alone implementation of ensuring clean-up callback which doesn't require sub classing
class AsyncDisposable extends Disposable implements IAsyncDisposable {
  final Completer _onDispose = new Completer();
  Future get future => _onDispose.future;

  AsyncDisposable._([dynamic callback()])
    : super(callback);

  /*
   * Creates a disposable instance which triggers (if provided) the supplied callback,
   * potentially with a completing value in either the callback or future completion
   */
  factory AsyncDisposable([dynamic callback()]) {
    // If the callback isn't defined then skip it
    if(callback == null) { return new AsyncDisposable._(null); }
    AsyncDisposable disposable = new AsyncDisposable._(callback);
    disposable.future
      .catchError((error) {
        //print('Exception raised in Async Callback: ${error}');
        //disposable._handleDisposeError(error);
      });
    return disposable;
  }

  void dispose() {
    try {
      super.dispose();
      if(!_onDispose.isCompleted) {
        _onDispose.complete();
      }
    } catch(ex) {
      _handleDisposeError(ex);
      throw ex;
    }
  }
  /*
   * Triggers the cleanup callback provided to the disposable
   * Dispose cannot be called multiple times on the same instance
   */
  void disposeWith(dynamic value) {
    try {
      super.dispose();
      if(!_onDispose.isCompleted){
        _onDispose.complete(value);
      }
    } catch(ex) {
      _handleDisposeError(ex);
      throw ex;
    }
  }

  /*
   * Triggers the cleanup callback provided to the disposable and passes along the error message
   * Dispose cannot be called multiple times on the same instance
   */
  void disposeError(dynamic error) {
    try {
      super.dispose();
      if(!_onDispose.isCompleted) {
        _onDispose.completeError(error);
      }
    } catch(ex) {
      _handleDisposeError(ex);
      throw ex;
    }
  }

  void _handleDisposeError(dynamic error) {
    _onDispose.completeError(error);
  }
}