part of disposable;

/*
 * Extends IDisposable to provide async disposal using Future and, as a side effect, also
 * adds support for disposeWith to dispose with a value and disposeError to indicate an
 * exception during disposal
 */
abstract class IAsyncDisposable implements IDisposable {
  /*
   *  Triggers cleanup on the target disposable and, presumably, passes the value to the future listener
   */
  void disposeWith(dynamic value);
  /*
   *  Triggers cleanup on the target disposable and, presumably, passes the error to the Future listener's error handler
   */
  void disposeError(dynamic error);
  /*
   *  Gets a Future which completes on dispose, potentially with a value if disposeWith was used
   */
  Future get future;
}