part of disposable;

/// Provides a method of encapsulating a series of actions which ensure proper disposal of
/// the disposable instance prior to exiting
void using(IDisposable disposable, void UsingFunction(IDisposable disposable)) {
  var caughtException = null;
  try {
    // Execute wrapped code
    UsingFunction(disposable);
  } catch(ex) {
    // Capture the wrapped code error
    caughtException = ex;
  } finally {
    if(caughtException == null) { // No exceptions during execution
      disposable.dispose(); // Just call dispose
    } else { // An exception was rasied during execution
      if(disposable is! IAsyncDisposable) {
        // Dispose the target disposable
        disposable.dispose();
      } else {
        // If disposable is Async then call disposeError
        disposable.disposeError(caughtException);
      }
      // Re-throw the exception
      throw caughtException;
    }
  }
}