part of disposable;

/// Provides a method of encapsulating a series of actions which ensure proper disposal of
/// the disposable instance prior to exiting but following the completion of a Future value
/// which is expected to be emitted from the contained code block
Future usingAsync(IDisposable disposable, dynamic UsingFunction(IAsyncDisposable disposable)) {
  Completer completer = new Completer();
  Future future = new Future(() {
    var caughtException = null;
    var result = null;
    try {
      // Execute wrapped code
      result = UsingFunction(disposable);
    } catch(ex) {
      // Capture the wrapped code error
      caughtException = ex;
    } finally {
      disposable.dispose();
      if(caughtException != null) {
        //completer.completeError(caughtException);
        throw caughtException;
        // return;
      } else {
        // Handle result
        return result;
      }
    }
  })
  .then((value) {
    if(completer.isCompleted) { return; }
    completer.complete(value);
  }, onError: (error) {
    completer.completeError(error);
  });

  return completer.future;
}