part of disposable_test;

disposable_tests() {

  group('-disposable-', () {

    test('not throw exception on dispose when no callback is provided, aka use noop instead of null', () {
      // Arrange
      IDisposable disposable = new Disposable();
      bool exceptionCaught = false;

      // Act
      try {
        disposable.dispose();
      } catch(ex) {
        exceptionCaught = true;
      }

      // Assert
      expect(exceptionCaught, false);
    });

    test('flip isDisposed when disposed', () {
      // Arrange
      IDisposable disposable = new Disposable();

      // Act
      expect(disposable.isDisposed, false);
      disposable.dispose();

      // Assert
      expect(disposable.isDisposed, true);
    });

    test('trigger callback when disposed', () {
      // Arrange
      bool disposed = false;
      IDisposable disposable = new Disposable(() { disposed = true; });

      // Act
      disposable.dispose();

      // Assert
      expect(disposed, true);
    });

    test('only trigger dispose method at most once', () {
      // Arrange
      int disposedCount = 0;
      var callback = () { disposedCount++; };
      IDisposable disposable = new Disposable(callback);

      // Act
      disposable.dispose();
      disposable.dispose();
      disposable.dispose();

      // Assert
      expect(disposedCount, 1);
    });

    test('not trap error if unhandled exception is raised in callback', () {
      // Arrange
      bool caughtError = false;
      var callback = () => throw 'error';
      IDisposable disposable = new Disposable(callback);

      // Act
      try {
        disposable.dispose();
      } catch(ex) {
        caughtError = true;
      }

      // Assert
      expect(caughtError, true);
    });

  });

}