part of disposable_test;

using_async_tests() {

  group('-using_async- should', () {

    test('allow nothing response from wrapped code without blocking', () {
      // Arrange
      bool disposed = false;
      IAsyncDisposable disposable = new AsyncDisposable(() => disposed = true);

      // Act
      usingAsync(disposable, (disposable) {
        expect(disposed, false, reason: 'should not have disposed inside the wrapped');
        // nothing...
      })
      .then((value) {
        // Assert
        expect(disposed, true, reason: 'should have disposed prior to completion without blocking');
      }, onError: (error) {
        expect(false, true, reason: 'should not complete with an exception: ${error}');
      })
      .then(expectAsync1((_) { }));

    });

    test('complete with onError from Future if exception is thrown in wrapped code', () {
      // Arrange
      bool disposed = false;
      IAsyncDisposable disposable = new AsyncDisposable(() => disposed = true);

      // Act
      usingAsync(disposable, (disposable) {
        expect(disposed, false, reason: 'should not have disposed inside the wrapped');
        throw 'error';
      })
      .then((value) {
        expect(false, true, reason: 'should complete with an exception if wrapped throws');
      }, onError: (error) {
        // Assert
        expect(error, 'error');
        expect(disposed, true, reason: 'should have disposed prior to completion with error: ${error}');
      })
      .then(expectAsync1((_) { }));
    });

    test('complete with onError from Future if exception is thrown in wrapped code and dispose', () {
      // Arrange
      bool disposed = false;
      IAsyncDisposable disposable = new AsyncDisposable(() => throw 'dispose error');
      // Act
      usingAsync(disposable, (disposable) {
        expect(disposed, false, reason: 'should not have disposed inside the wrapped');
        throw 'error';
      })
      .then((value) {
        print('got da value... ${value}');
        expect(false, true, reason: 'should complete with an exception if both wrapped and dispose throw');
      }, onError: (error) {
        // Assert
        expect(error, 'dispose error');
      })
      .then(expectAsync1((_) { }));
    });

    test('complete with onError from Future if exception is thrown in dispose but not in wrapped code', () {
      // Arrange
      bool disposed = false;
      IAsyncDisposable disposable = new AsyncDisposable(() => throw 'dispose error');

      // Act
      usingAsync(disposable, (disposable) {
        expect(disposed, false, reason: 'should not have disposed inside the wrapped');
        // nothing...
      })
      .then((value) {
        expect(false, true, reason: 'should complete with an exception');
      }, onError: (error) {
        // Assert
        expect(error, 'dispose error');
      })
      .then(expectAsync1((_) { }));
    });
  });

}