part of disposable_test;

using_tests() {

  group('-using- should', () {

    test('trigger dispose when exiting using statement', () {
      // Arrange
      bool disposed = false;
      Disposable disposable = new Disposable(() => disposed = true);

      // Act
      using(disposable, (IDisposable disposable) {
        expect(disposed, false);
      });

      // Assert
      expect(disposed, true);
    });

    test('still trigger dispose if wrapped block throws exception', () {
      // Arrange
      bool disposed = false;
      Disposable disposable = new Disposable(() => disposed = true);
      // Act
      try {
        using(disposable, (IDisposable disposable) {
          throw 'error';
        });
      } catch(ex) { }

      // Assert
      expect(disposed, true);
    });

    test('rethrow caught exception after disposing', () {
      // Arrange
      bool disposed = false;
      bool caughtError = false;
      var error = null;
      Disposable disposable = new Disposable(() => disposed = true);

      // Act
      try {
        using(disposable, (IDisposable disposable) {
          throw 'error';
        });
      } catch(ex) {
        caughtError = true;
        error = ex;
      }

      // Assert
      expect(disposed, true);
      expect(caughtError, true, reason: 'should have been re-thrown by using block');
      expect(error, 'error');
    });

    test('trigger Future.completeError if disposable is IAsyncDisposable and exception is thrown in wrapped block', () {
      // Arrange
      bool disposed = false;
      bool futureCalled = false;
      var error = null;
      IAsyncDisposable disposable = new AsyncDisposable(() => disposed = true);

      var future = disposable.future
          .then((_) => futureCalled = true)
          .catchError((ex) => error = ex)
          .then(expectAsync1((_) {
            expect(futureCalled, false);
            expect(error, 'error', reason: 'if the future error handler was triggered this will not be null');
          }));

      // Act
      try {
        using(disposable, (IAsyncDisposable disposable) {
          throw 'error';
        });
      } catch(ex) { }

      expect(disposed, true);
    });
  });
}